import string
from fabric.connection import Connection
from tif.fabric import cli
from tif.fabric.logger import Logger
from tif.fabric.CommandPrefix import CommandPrefix
from tif.cli.options import Options

def version(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git --version"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def get_remote_url(c: Connection, repo_dir: string, command_prefix=""):
    with c.cd(repo_dir):
        command = "git config --get remote.origin.url"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def clone_repo(c : Connection, clone_dir: string, repo: string, command_prefix = ""):
    with c.cd(clone_dir):
        command = "git clone {} .".format(repo)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def switch_branch(c: Connection, repo_dir: string, command_prefix = ""):
    with c.cd(repo_dir):
        branch(c, repo_dir, command_prefix)
        sbranch = cli.prompt(">>> Enter branch to switch to: ")
        if (sbranch):
            command = "git checkout {}".format(sbranch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def switch_to_branch(c: Connection, repo_dir: string, branch : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git checkout {}".format(branch)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def new_branch(c: Connection, repo_dir: string, command_prefix = ""):
    with c.cd(repo_dir):
        new_branch = cli.prompt(">>> Enter new branch name: ")
        if (new_branch):
            command = "git checkout -b {}".format(new_branch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def delete_local_branch(c: Connection, repo_dir: string, command_prefix = ""):
    with c.cd(repo_dir):
        delete_branch = cli.prompt(">>> Enter branch to delete: ").strip()
        if (delete_branch):
            command = "git branch --delete {}".format(delete_branch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")
 
def branch(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git branch"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def branch_all(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git branch --all"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def current_branch(c : Connection, repo_dir : string, command_prefix = "") -> string :
    with c.cd(repo_dir):
        command = "git rev-parse --abbrev-ref HEAD"
        Logger().log("Running command '{}'".format(command))
        branch = c.run(CommandPrefix(command, command_prefix).prefix_command(), hide=True)
        return branch.stdout.strip()

def pull(c : Connection, repo_dir : string,command_prefix = "", oauth2_url = None):
    with c.cd(repo_dir):
        branch = current_branch(c, repo_dir, command_prefix)
        confirm = cli.cli_confirm("You are about to pull the remote branch {}. Are you sure?".format(branch))
        if (confirm == "y"):
            if oauth2_url:
                command = "git pull {} {}".format(oauth2_url, branch)
            else:
                command = "git pull origin {}".format(branch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
        else:
            cli.puts("!!! Aborted")

def pull_branch(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        branch = current_branch(c, repo_dir, command_prefix)
        origin_branch = cli.prompt(">>> Enter branch to pull from origin: ")
        confirm = cli.cli_confirm("You are about to pull the remote branch {} into {}. Are you sure?".format(origin_branch, branch))
        if (confirm == "y"):
            command = "git pull origin {}".format(origin_branch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
        else:
            cli.puts("!!! Aborted")

def origin_merge(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        cli.puts("** Fetching origin...")
        fetch(c, repo_dir, command_prefix)
        branch(c, repo_dir, command_prefix)
        local_branch = current_branch(c, repo_dir, command_prefix)
        origin_branch = cli.prompt(">>> Enter origin branch to merge: ")
        if (local_branch and origin_branch):
            confirm = cli.cli_confirm("You are about to merge the origin branch {} into local branch {}. Are you sure?".format(origin_branch, local_branch))
            if (confirm == "y"):
                command = "git merge origin/{}".format(origin_branch.strip())
                Logger().log("Running command '{}'".format(command))
                c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
            else:
                cli.puts("!!! Aborted")
        else:
            cli.puts("!!! Aborted")

def push(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        branch = current_branch(c, repo_dir, command_prefix)
        confirm = cli.cli_confirm("You are about to push to origin branch {}. Are you sure?".format(branch))
        if (confirm == "y"):
            command = "git push origin {}".format(branch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
        else:
            cli.puts("!!! Aborted")

def force_push(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        branch = current_branch(c, repo_dir, command_prefix)
        confirm = cli.cli_confirm("You are about to force push to origin branch {}. Are you sure?".format(branch))
        if (confirm == "y"):
            command = "git push -f origin {}".format(branch)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)
        else:
            cli.puts("!!! Aborted")

def fetch(c : Connection, repo_dir : string, command_prefix = "", oauth2_url = None, branch = None):
    with c.cd(repo_dir):
        command = "git fetch"
        if oauth2_url:
            command = command + " " + oauth2_url
        if branch:
            command = command + " " + branch
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def status(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git status"
        Logger().log(message="Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def diff(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        file = cli.prompt(">>> Enter file to diff: ")
        command = "git diff {}".format(file)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def file_checkout(c : Connection, repo_dir : string, command_prefix = "", remote = True):
    with c.cd(repo_dir):
        status(c, repo_dir, command_prefix)
        if remote:
            file = Options().remote_file_chooser(c, repo_dir, input_text="Navigate to file to checkout (CTRL+c to abort): ")
        else:
            file = Options().local_file_chooser(repo_dir, input_text="Navigate to file to checkout (CTRL+c to abort): ")
        if(file):
            command = "git checkout -- {}".format(file)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def add_iteractive(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git add -i"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def commit(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git commit"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def stash(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git stash"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def stash_list(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git stash list"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def stash_drop(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git stash drop"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def stash_pop(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git stash pop"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def reset_head(c : Connection, repo_dir : string, command_prefix = ""):
    with c.cd(repo_dir):
        command = "git reset --hard HEAD"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def remove(c : Connection, repo_dir : string, command_prefix = ""):
     with c.cd(repo_dir):
        file = cli.prompt(">>> Enter file to remove. Type 'exit' to finish the operation: ")
        while file not in ["exit"]:
            command = "git rm {}".format(file)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
            status(c, repo_dir, command_prefix)
            file = cli.prompt(">>> Enter file to remove. Type 'exit' to finish the operation: ")
        status(c, repo_dir, command_prefix)

def rename(c : Connection, repo_dir : string, command_prefix = ""):
     with c.cd(repo_dir):
        old_file = cli.prompt(">>> Enter file to rename: ")
        new_file = cli.prompt(">>> Enter new filename: ")
        command = "git mv {} {}".format(old_file, new_file)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())
        status(c, repo_dir, command_prefix)

def log(c : Connection, repo_dir : string, logs = "-5", command_prefix = ""):
    with c.cd(repo_dir):
        command = "git log -p {}".format(logs)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def reset_commit(c : Connection, repo_dir : string, logs = "-5", command_prefix = ""):
    with c.cd(repo_dir):
        command = "git reflog"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())
        hash = cli.prompt(">>> Enter commit hash to be reset: ")
        if (hash):
            confirm = cli.cli_confirm("You are about to execute git reset --hard {}. Are you sure?".format(hash))
            if (confirm == 'y'):
                command = "git reset --hard {}".format(hash)
                Logger().log("Running command '{}'".format(command))
                c.run(CommandPrefix(command, command_prefix).prefix_command())
            else:
                cli.puts("!!! Aborted")
        else:
            cli.puts("!!! Aborted")



import string
import emoji
from . import Colors

c = Colors.Colors()

def confirm(question):
    return input(c.get(question))

def prompt(text):
    return input(c.get(text))

def puts(text):
    print(c.get(text))

def puts_hide(text) -> string:
    return c.get(text)

def cli_confirm(message):
    value = confirm("*** {} {} Y/n ".format(emoji.emojize(':backhand_index_pointing_right:'), message))
    while value not in ["Y", "n"]:
        value = confirm("*** {} {} Y/n ".format(emoji.emojize(':backhand_index_pointing_right:'),message))
    return value.lower()  



import string
import emoji
from tif.fabric import cli

class Logger:
    def __init__(self) -> None:
        pass

    def log(self, message : string):
        """
        Outputs white message to console with :bell: emoji
        """
        cli.puts("** {} {}".format(emoji.emojize(':bell:'), message))

    def info(self, message : string):
        """
        Outputs white message to console with :memo: emoji
        """
        cli.puts("!! {} {}".format(emoji.emojize(':memo:'), message))
    
    def error(self, message : string):
        """
        Outputs red message to console with :red_exclamation_mark: emoji
        """
        cli.puts("error: {} {}".format(emoji.emojize(':red_exclamation_mark:'), message))

    def success(self, message : string):
        """
        Outputs green message to console
        """
        cli.puts(".:~ {}".format(message))
    
    def log_custom(self, color_key : string, message : string, emoji_string : None):
        """
        Outputs a custom message to console. Allows to specify a color and emoji for the message
        """
        if emoji:
            cli.puts("{} {} {}".format(color_key, emoji.emojize(emoji_string), message))
        else:
            cli.puts("{} {}".format(color_key, message))


import string
from fabric.connection import Connection
from tif.fabric.logger import Logger

class Cloud:
    def __init__(self, docker_dir : string, magento_root: string) -> None:
        self.docker_dir = docker_dir
        self.magento_root = magento_root

    def docker_compose_deploy_run(self, c : Connection, command : string):
        docker_compose_command = "docker-compose run --rm deploy magento-command {}".format(command)
        with c.cd(self.docker_dir):
            Logger().log("Running command '{}'".format(docker_compose_command))
            c.run(docker_compose_command, pty=True)

    def environment_checkout(self, c : Connection, environment : string):
        checkout_command = "magento-cloud environment:checkout {}".format(environment)
        with c.cd(self.magento_root):
            Logger().log("Running command '{}'".format(checkout_command))
            c.run(checkout_command, pty=True)

    def start_mutagen(self, c : Connection):
        with c.cd(self.magento_root):
            Logger().log("Running command '{}'".format("./mutagen.sh"))
            c.run("./mutagen.sh", pty=True)

    def ece_patches_apply(self, c : Connection):
        with c.cd(self.docker_dir):
            command = "docker-compose run --rm deploy php ./vendor/bin/ece-patches apply"
            Logger().log("Running command '{}'".format(command))
            result = c.run(command, pty=True, warn=True)
            if result.return_code == 0:
                Logger().success("ece-patches apply command run successfully")
            else:
                Logger().error("ece-patches apply command exited with error")

    def exec_db_query(self, c : Connection, container : string, db_user : string, db_password : string, db_name : string, query: string, capture = False):
        with c.cd(self.docker_dir):
            command = 'docker exec {} mysql -u{} -p{} {} -e "{}"'.format(container, db_user, db_password, db_name, query)
            Logger().log("Running command '{}'".format(command))
            if capture == True:
                result = c.run(command, hide=True, warn=True)
                return result.stdout
            else:
                c.run(command, pty=True)

    def cloud_deploy(self, c : Connection):
        with c.cd(self.docker_dir):
            command = "docker-compose run --rm deploy cloud-deploy"
            Logger().log("Running command '{}'".format(command))
            c.run(command, pty=True)

    def cloud_post_deploy(self, c : Connection):
        with c.cd(self.docker_dir):
            command = "docker-compose run --rm deploy cloud-post-deploy"
            Logger().log("Running command '{}'".format(command))
            c.run(command, pty=True)


    def magento_cloud_command_test(self, c : Connection) -> int:
        with c.cd(self.magento_root):
            command = 'magento-cloud'
            Logger().log("Running command '{}'".format(command))
            result = c.run(command, hide=True, warn=True)
            return result.return_code



import re
import emoji

def _wrap_with(code):
    def inner(text, bold=False):
        c = code
        if bold:
            c = "1;%s" % c
        return "\033[%sm%s\033[0m" % (c, text)
    return inner

red = _wrap_with('31')
green = _wrap_with('32')
yellow = _wrap_with('33')
blue = _wrap_with('34')
magenta = _wrap_with('35')
cyan = _wrap_with('36')
white = _wrap_with('37')

class Colors:
        def __init__(self):
            pass
        def get(self, text):
            if re.match(r'^\>\>\>', text):
                return cyan(text[4:])
            if re.match(r'^\-\-\-', text):
                return magenta(text[4:])
            if re.match(r'^\*\*\s', text):
                return white(text[3:])
            if re.match(r'^\*\*\*', text):
                return yellow(text[4:])
            if re.match(r'^\[\d{1,}\]', text):
                return yellow(text)
            if re.match(r'^\!\!\!', text):
                return red(emoji.emojize(':bell_with_slash:') + text[3:])
            if re.match(r'^error:', text):
                return red(text[7:])
            if re.match(r'^\!\!\s', text):
                return blue(text[3:])
            if re.match(r'^\.:\~', text):
                return green(emoji.emojize(':check_box_with_check:') + " " + text[3:])


import string
from fabric.connection import Connection
from tif.fabric.CommandPrefix import CommandPrefix
from tif.fabric.logger import Logger

def version(c : Connection, install_dir : string, command_prefix = ""):
    with c.cd(install_dir):
        command = "composer -V"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def install(c : Connection, install_dir : string, command_prefix = ""):
    with c.cd(install_dir):
        command = "composer install"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def install_with_flags(c : Connection, install_dir : string, flags = "", command_prefix = ""):
    with c.cd(install_dir):
        command = "composer install {}".format(flags)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def update(c : Connection, install_dir : string, command_prefix = ""):
    with c.cd(install_dir):
        command = "composer update"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)



import string
import docker
import re
from invoke import run
from tif.fabric import cli
from tif.fabric.logger import Logger

class Service:
    def __init__(self, docker_dir : string) -> None:
        self.docker_dir = docker_dir

    def container_name_search(self, search : string) -> string:
        client = docker.from_env()
        containers = client.containers.list()
        for container in containers:
            if re.search(search, container.name):
                return container.name
        raise Exception("No {} container match".format(search))

    def containers_dictionary(self) -> dict:
        """
        Returns dictionary with numbers as key and container's name as value
        """
        containers_dict = {}
        client = docker.from_env()
        containers = client.containers.list()
        index = 1
        for container in containers:
            containers_dict[str(index)] = container.name
            index = index + 1
        return containers_dict

    def exec(self, options, command = None, confirm_prompt = True, exec_options = "-it"):
        """
        Exec a command in chosen container. Expect Options object from tif.cli.options module
        """
        container_choice = options.docker_container_chooser(self, confirm_prompt=confirm_prompt)
        if not command:
            command = cli.prompt(">>> Enter command to execute: ")
        command = self.command(container_choice, command.strip(), exec_options)
        run(command, pty=True)

    def command(self, container : string, command : string, exec_options = "-it") -> string:
        """
        Returns formatted command to execute on a docker container
        """
        command = "docker exec {} {} bash -c \"{}\"".format(exec_options, container, command)
        Logger().log("Running command '{}'".format(command))
        return command

    def composeExec(self, yml_location: string, container : string, command : string, exec_options = "-it") -> string:
        """
        Returns formatted command to execute on a docker container
        """
        command = "cd {} && docker-compose exec {} {} bash -c \"{}\"".format(yml_location, exec_options, container, command)
        Logger().log("Running command '{}'".format(command))
        return command

    def containerBash(self, container : string):
        """
        Logs into a container bash
        """
        docker_compose_command = "docker-compose exec {} bash".format(container)
        command = "cd {} && {}".format(self.docker_dir, docker_compose_command)
        Logger().log("Running command '{}'".format(docker_compose_command))
        run(command, pty=True)
    
    def restart_container(self, container : string):
        """
        Restart a container
        """
        command = "docker restart {}".format(container)
        Logger().log("Running command '{}'".format(command))
        run(command, pty=True)


import string
from fabric.connection import Connection
from tif.fabric import cli
from datetime import datetime

def import_db(c : Connection, dump_location : string):
    with c.cd(dump_location):
        c.run("ls")
        user = cli.prompt(">>> Enter DB user: ")
        db = cli.prompt(">>> Enter DB name: ")
        dump = cli.prompt(">>> Enter dump filename: ")
        if (user and db and dump):
            c.run("mysql -u{} -p {} < {}".format(user, db, dump), pty=True)
            cli.puts(".:~ Import finished: {0}".format(dump))
        else:
            cli.puts("!!! Aborted")

def dump(c : Connection, host : string, user : string, database : string, dump_location : string, defaults_extra_file = None):
    with c.cd(dump_location):
        now = datetime.now()
        date_and_time = now.strftime("%m-%d-%Y_%H:%M:%S")
        dump = "{}_{}.sql.gz".format(database, date_and_time)
        if defaults_extra_file:
            result = c.run("mysqldump --defaults-extra-file={} -h{} -u{} {} | gzip > {}".format(defaults_extra_file, host, user, database, dump), warn=True)
        else:
            result = c.run("mysqldump -h{} -u{} -p {} | gzip > {}".format(host, user, database, dump), warn=True)
        if(result.return_code == 0):
            cli.puts(".:~ Database dump successfully completed and stored in {} directory: {}".format(dump_location, dump))
        else:
            cli.puts("!!! Aborted with errors")


import string
import os
from fabric.connection import Connection
from fabric.transfer import Transfer
from tif.fabric import cli
from tif.fabric.logger import Logger
from tif.fabric.CommandPrefix import CommandPrefix
from tif.cli.options import Options
from python_hosts import Hosts, HostsEntry

def gunzip(c : Connection , dir : string, listFiles = True, command_prefix = ""):
    with c.cd(dir):
        if (listFiles):
            command = "ls"
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        zip = cli.prompt(">>> Enter file to gunzip: ")
        if (zip):
            command = "gunzip {}".format(zip)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def ls(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        command = "ls"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def mv(c : Connection, dir : string, filename : string, mv_filename : string, command_prefix = ""):
     with c.cd(dir):
        command = "mv {} {}".format(filename, mv_filename)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def rsync(c : Connection, dir : string, command : string, command_prefix = ""):
    with c.cd(dir):
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def mount(c : Connection, command : string, command_prefix = ""):
    Logger().log("Running command '{}'".format(command))
    c.run(CommandPrefix(command, command_prefix).prefix_command())

def dfh(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        command = "df -h"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def grep(c : Connection, dir : string, command : string, search : string, command_prefix = "") -> int:
    with c.cd(dir):
        command = "{} | grep -q '{}'".format(command, search)
        Logger().log("Running command '{}'".format(command))
        result = c.run(CommandPrefix(command, command_prefix).prefix_command(), hide=True, warn=True)
        return result.return_code


def mkdir(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        curr_dir = c.run("pwd")
        new_dir = cli.prompt(">>> Enter directory to create in {} directory: ".format(curr_dir.stdout.strip()))
        if (new_dir):
            command = "mkdir {}".format(new_dir)
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command())
        else:
            cli.puts("!!! Aborted")

def ls_tmp(c : Connection, command_prefix = ""):
    with c.cd("/tmp"):
        command = "ls -la"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def rm(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        command = "ls"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())
        to_delete = cli.prompt(">>> Enter directory/file to delete: ")
        if c.run(CommandPrefix('test -d {0}'.format(to_delete), command_prefix).prefix_command(), warn=True):
            confirm = cli.cli_confirm("You are about to delete the directory {0}. Are you sure?".format(to_delete))
            if (confirm == "y"):
                command = "rm -rf {0}".format(to_delete.strip())
                Logger().log("Running command '{}'".format(command))
                c.run(CommandPrefix(command, command_prefix).prefix_command())
            else:
                cli.puts("!!! Aborted")
        elif c.run(CommandPrefix('test -f {}'.format(to_delete), command_prefix).prefix_command(), warn=True):
            confirm = cli.cli_confirm("You are about to delete the file {0}. Are you sure?".format(to_delete))
            if (confirm == "y"):
                command = "rm -f {}".format(to_delete.strip())
                Logger().log("Running command '{}'".format(command))
                c.run(CommandPrefix(command, command_prefix).prefix_command())
            else:
                cli.puts("!!! Aborted")
        else:
            cli.puts("!!! Aborted operation. {} not found.".format(to_delete))

def vim(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        file = cli.prompt(">>> Enter file to edit in vim: ")
        command = "vim {}".format(file.strip())
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def ls_dir(c : Connection, dir : string, command_prefix = "", prompt_for_dir = True):
    if (prompt_for_dir):
        ls_dir = cli.prompt(">>> Enter directory to list: ")
    else:
        ls_dir = ""
    with c.cd(dir + "/" + ls_dir.strip()):
        command = "ls -la"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def ls_la(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        command = "ls -la"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def find_mount(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        command = "findmnt -T ."
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def mount_column(c : Connection, dir : string, command_prefix = ""):
    with c.cd(dir):
        command = "mount | column -t"
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def get(c : Connection, from_dir : string, to_dir : string):
    t = Transfer(c)
    remote_file = Options().remote_file_chooser(c, dir = from_dir, absolute_path=True,input_text="Navigate to file to transfter (CTRL+c to abort): ")
    local_dir = Options().local_directory_chooser(input_text="Navigate to local directory for the transfter (CTRL+c to abort): ", absolute_path=True)
    local_fileName = cli.prompt(">>> Enter local file name to be tansferred in {}: ".format(local_dir))
    t.get(remote_file, local_dir + os.sep + local_fileName)
    cli.puts(".:~ Done!")

def put(c : Connection, remote_working_dir = None):
    local_file = cli.prompt(">>> Enter local file path to upload (relative to fabfile location): ")
    remote_file = cli.prompt(">>> Enter remote file path to upload (relative to home directory or specified working dir): ")
    t = Transfer(c)
    if (remote_working_dir):
        remote_dir = remote_working_dir + '/' + remote_file
        cli.puts(">>> Uploading to {}".format(remote_dir))
        t.put(local=local_file, remote=remote_dir)
    else:
         t.put(local=local_file, remote=remote_file)

def cat(c : Connection, dir : string, command_prefix=""):
        with c.cd(dir):
            file = Options().remote_file_chooser(c, dir, input_text="Navigate to file to cat (CTRL+c to abort): ")
            command = "cat {}".format(file.strip())
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def cat_and_grep(c : Connection, dir : string, file : string, grep : string, command_prefix=""):
        with c.cd(dir):
            command = "cat {} | grep '{}'".format(file.strip(), grep.strip())
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def cat_file(c : Connection, dir : string, file : string, command_prefix=""):
        with c.cd(dir):
            command = "cat {}".format(file.strip())
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def cat_tmp_file(c : Connection, command_prefix=""):
        with c.cd("/tmp"):
            ls_tmp(c, command_prefix=command_prefix)
            file = cli.prompt(">>> Enter file to cat: ")
            command = "cat {}".format(file.strip())
            Logger().log("Running command '{}'".format(command))
            c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def set_files_permissions(c : Connection, dir : string, path : string, chmod : string, command_prefix = ""):
    with c.cd(dir):
        command = "sudo -s && find {} -type d -exec chmod {} {{}} \;".format(path, chmod)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command(), pty=True)

def chown(c : Connection, dir : string, owner : string, group : string, file : string, command_prefix = ""):
    with c.cd(dir):
        command = "chown {}:{} {}".format(owner, group, file)
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def tail_f(c : Connection, dir : string, file : string, command_prefix = ""):
    with c.cd(dir):
        command = "tail -f {}".format(file.strip())
        Logger().log("Running command '{}'".format(command))
        c.run(CommandPrefix(command, command_prefix).prefix_command())

def etc_hosts_add_entry(c : Connection, address : string, names : tuple, entry_type='ipv4', hosts_file='/etc/hosts'):
    command = "sudo chmod 766 {}".format(hosts_file)
    Logger().log("Running command '{}'".format(command))
    c.run(command, pty=False)
    hosts = Hosts()
    new_entry = HostsEntry(entry_type, address= address, names = names)
    hosts.add([new_entry])
    Logger().log("Adding to {} entry '{} {}'".format(hosts_file, address, names))
    hosts.write()
    command = "sudo chmod 644 {}".format(hosts_file)
    Logger().log("Running command '{}'".format(command))
    c.run(command, pty=False)

def etc_hosts_remove_entry(c : Connection, address : string, hosts_file='/etc/hosts'):
    command = "sudo chmod 766 {}".format(hosts_file)
    Logger().log("Running command '{}'".format(command))
    c.run(command, pty=False)
    hosts = Hosts()
    hosts.remove_all_matching(address=address)
    Logger().log("Removing from {} entry for address '{}'".format(hosts_file, address))
    hosts.write()
    command = "sudo chmod 644 {}".format(hosts_file)
    Logger().log("Running command '{}'".format(command))
    c.run(command, pty=False)